searchNodes=[{"doc":"Shoehorn helps you handle OTP application failures Shoehorn needs to be enabled when building your project. See the Shoehorn README.md for usage.","ref":"Shoehorn.html","title":"Shoehorn","type":"module"},{"doc":"Default handler that ignores all events","ref":"Shoehorn.DefaultHandler.html","title":"Shoehorn.DefaultHandler","type":"module"},{"doc":"A behaviour module for implementing handling of failing applications A Shoehorn.Handler is a module that knows how to respond to specific applications going down. There are two types of failing applications. The first is an application that fails to initialize and the second is an application that stops while it is running. Example The Shoehorn.Handler behaviour requires developers to implement two callbacks. The init callback sets up any state the handler needs. The application_started callback is called when an application starts up. The application_exited callback processes the incoming failure and replies with the reaction that Shoehorn should take in case of application failure. defmodule Example.ShoehornHandler do @behaviour Shoehorn.Handler @impl Shoehorn.Handler def init ( _opts ) do { :ok , %{ restart_counts : 0 } } end @impl Shoehorn.Handler def application_started ( app , state ) do { :continue , state } end @impl Shoehorn.Handler def application_exited ( :non_essential_app , _reason , state ) do { :continue , state } end def application_exited ( :essential_app , _reason , %{ restart_counts : restart_counts } = state ) when restart_counts &lt; 2 do # repair actions to make before restarting Application . ensure_all_started ( :essential_app ) { :continue , %{ state | restart_counts : restart_counts + 1 } } end def application_exited ( _ , state ) do { :halt , state } end end We initialize our Shoehorn.Handler with a restart count for state by calling init with the configuration options from our shoehorn config. When we have an application startup, we will put a message on the console to notify the developer. When we have a non-essential application fail we return :continue to inform the system to keep going like nothing happened. We restart the essential application of our system two times, and then we tell the system to halt if starting over wasn't fixing the system.","ref":"Shoehorn.Handler.html","title":"Shoehorn.Handler","type":"behaviour"},{"doc":"Callback for handling application crashes Called with the application name, cause, and the handler's state. It must return a tuple containing the reaction that the Shoehorn should take, and the new state of the handler. The code that you execute here can be used to notify or capture some information before halting the system. This information can later be used to recreate the issue or debug the problem causing the application to exit. Use application_exited as a place for a last-ditch effort to fix the issue and restart the application. Ideally, capture some information on the system state, and solve it upstream. Shoehorn restarts should be used as a splint to keep a critical system running. The default implementation returns the previous state, and a :halt reaction. Callback Timeout This callback is limited in the amount of time it has to execute by a shutdown timer. If the callback does not return within the defined shutdown time, the node is instructed to halt. The default shutdown time is 30 seconds but this value can be changed in the application config.","ref":"Shoehorn.Handler.html#c:application_exited/3","title":"Shoehorn.Handler.application_exited/3","type":"callback"},{"doc":"Callback for handling application starts Called with the application name, and the handler's state. It must return a tuple containing the reaction that the Shoehorn should take, and the new state of the handler. def application_exited ( :essential_app , _reason , state ) do # repair actions to make before restarting # notify someone of the crash and the details # log debug data Application . ensure_all_started ( :essential_app ) { :continue , %{ state | restart_counts : restart_counts + 1 } } end The default implementation returns unchanged state, and a :continue reaction.","ref":"Shoehorn.Handler.html#c:application_started/2","title":"Shoehorn.Handler.application_started/2","type":"callback"},{"doc":"","ref":"Shoehorn.Handler.html#init/1","title":"Shoehorn.Handler.init/1","type":"function"},{"doc":"Callback to initialize the handle The callback will receive a keyword list of all the configuration for shoehorn. The callback must return a tuple of {:ok, state} . Where state is the initial state of the handler. The system will halt if the return is anything other than :ok .","ref":"Shoehorn.Handler.html#c:init/1","title":"Shoehorn.Handler.init/1","type":"callback"},{"doc":"","ref":"Shoehorn.Handler.html#invoke/3","title":"Shoehorn.Handler.invoke/3","type":"function"},{"doc":"","ref":"Shoehorn.Handler.html#invoke/4","title":"Shoehorn.Handler.invoke/4","type":"function"},{"doc":"The cause that is firing the handler","ref":"Shoehorn.Handler.html#t:cause/0","title":"Shoehorn.Handler.cause/0","type":"type"},{"doc":"","ref":"Shoehorn.Handler.html#t:opts/0","title":"Shoehorn.Handler.opts/0","type":"type"},{"doc":"The reaction letting Shoehorn know what to do :continue - keep the system going like nothing happened :halt - stop the application and bring the system down","ref":"Shoehorn.Handler.html#t:reaction/0","title":"Shoehorn.Handler.reaction/0","type":"type"},{"doc":"","ref":"Shoehorn.Handler.html#t:t/0","title":"Shoehorn.Handler.t/0","type":"type"},{"doc":"Shoehorn helps you handle OTP application failures","ref":"readme.html","title":"Shoehorn","type":"extras"},{"doc":"By default, the Erlang VM exits when OTP applications unexpectedly stop. This can happen if an application's Application.start/2 callback crashes or if a GenServer crashes repeatedly and takes down the application's supervision tree. Either way, recovery needs to happen outside of the Erlang VM. Shoehorn provides a way of handling this inside the Erlang VM to allow you to debug, restart an application, switch to a recovery mode, or something else of your choosing. It does this by creating a custom release start script ( shoehorn.boot ) and exposing the Shoehorn.Handler behaviour for your code to decide what to do. The custom release start script turns off the default OTP application mode that exits the VM on unexpected errors and orders application starts to make sure that the handler is available. Shoehorn has another benefit of letting you influence the OTP application start order. Dependencies still determine the overall ordering, but it's possible to sort applications earlier via Shoehorn's :init option. This can let you improve the apparent release startup time on slow platforms.","ref":"readme.html#motivation","title":"Shoehorn - Motivation","type":"extras"},{"doc":"Run mix release.init on your project and then add shoehorn to your mix releases configuration in the mix.exs (replace :simple_app ): def project do [ ... releases : releases ( ) ] end def releases do [ simple_app : [ steps : [ &amp; Shoehorn.Release . init / 1 , :assemble ] ] ] end defp deps do [ { :shoehorn , &quot;~&gt; 0.9.0&quot; } ] end end Create a release: mix release Next, take a look at the start script so that you can see how your application will now be started and how it compares to the default startup.script . Open _build/dev/rel/simple_app/releases/0.1.0/shoehorn.script and go to the end. You should see something like the following: { progress , applications_loaded } , { apply , { application , start_boot , [ kernel , permanent ] } } , { apply , { application , start_boot , [ stdlib , permanent ] } } , { apply , { application , start_boot , [ compiler , permanent ] } } , { apply , { application , start_boot , [ elixir , permanent ] } } , { apply , { application , start_boot , [ logger , permanent ] } } , { apply , { application , start_boot , [ crypto , permanent ] } } , { apply , { application , start_boot , [ shoehorn , permanent ] } } , { apply , { application , start_boot , [ sasl , permanent ] } } , { apply , { application , start_boot , [ simple_app , temporary ] } } , { progress , started } This shows the order that applications will be started and their mode. Applications marked permanent will exit the VM if they stop expectantly. Shoehorn will change as much as it can to temporary so that it (and by extension, you) can control what happens. To start your release using the shoehorn boot script, run: RELEASE_BOOT_SCRIPT=shoehorn _build/dev/rel/simple_app/bin/simple_app start_iex It should work as expected with the possible exception that the Erlang VM won't exit for any of the OTP applications marked temporary . Now let's configure shoehorn to do something more interesting by adding some minimal configuration. This is hypothetical unless you're using Nerves: # config/config.exs config :shoehorn , init : [ :nerves_runtime , :nerves_pack ] Shoehorn will generate a release script that starts :nerves_runtime and its dependencies as soon as it can. Then it will start :nerves_pack and its dependencies. Then it will start the remainder of the applications in the project. Inspect the shoehorn.script file in the release directory to verify this. Use the init application list to prioritize OTP applications that are needed for early on or for error recovery. In the example above, we initialize the runtime, bring up the network (in :nerves_pack ), and ensure that we can receive new firmware updates. Now, if simple_app fails to start, the device would still be in a state where it can receive new firmware over the network.","ref":"readme.html#usage","title":"Shoehorn - Usage","type":"extras"},{"doc":"The Erlang VM will respond to application failures differently, depending on the mode specified when the application started. The modes are: :permanent - if the application terminates, all other applications and the entire node are also terminated. :transient - if the application terminates with :normal reason , it is reported but no other applications are terminated. However, if the application terminates abnormally, all other applications and the entire node are also terminated. :temporary - if the application terminates, it is reported but no other applications are terminated (the default behaviour). Unless overridden in the Mix release using the :applications option , Shoehorn most applications as :temporary and monitors application events by registering with the Erlang error_logger . Application start and exit events will attempt to execute a callback to the configured Shoehorn.Handler module. By default, the module Shoehorn.DefaultHandler will be called. This module is configured to continue the Erlang VM if any OTP application were to exit, for any reason. In production, you may want to customize the action on failure so you can gather forensics or perform updates to the node. You can do this by overriding the handler in the prod env of your application config. # config/prod.exs config :shoehorn , handler : SimpleApp.ShoehornHandler More advanced failure cases can be handled by providing your own module that implements the Shoehorn.Handler behaviour. For example, the Erlang :ssh application used to exit when subjected to a brute force attack (this seems like it has been fixed). Instead of the default production behaviour of forcing the node to restart, we can restart the application. defmodule Example.RestartHandler do @behavior Shoehorn.Handler def init ( _opts ) do { :ok , :no_state } end def application_started ( _app , state ) do { :continue , state } end def application_exited ( :ssh , _reason , state ) do Logger . error ( &quot;Stop bothering ssh!&quot; ) Process . sleep ( 1000 ) Application . ensure_all_started ( :ssh ) { :continue , state } end def application_exited ( app , _reason , state ) do Logger . error ( &quot;Application stopped! \#{ inspect ( app ) } \#{ inspect ( state ) } &quot; ) { :halt , state } end end The application_exited/3 callback is limited in the amount of time is has to execute by setting a shutdown timer. If the callback does not return within the defined shutdown time, the node is instructed to halt. The default shutdown time is 30 seconds but this value can be changed in the application config: # config/config.exs config :shoehorn , shutdown_timer : 50_000 # 50 Seconds Have a look at the example application for more info on implementing custom strategies.","ref":"readme.html#handling-application-failures","title":"Shoehorn - Handling application failures","type":"extras"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"This is a major update to Shoehorn that includes breaking changes : The :init configuration option only supports applications now. MFAs are no longer supported and moved to runtime.exs or an Application.start callback. References to use Shoehorn.Handler need to be updated to @behaviour Shoehorn.Handler . This may require implementing additional functions. Elixir 1.9 is no longer supported. Please update to Elixir 1.10 or later. The main update to Shoehorn is to move all application startup to the boot script. This noticeably improves boot time on many Nerves platforms due to boot scripts being able to load files without traversing the entire Erlang module path list. These traversals are amazingly slow (sometimes seconds) due to a combination of SquashFS slowness in this area and slow overall IO. Using boot scripts to load all applications has some important improvements in addition to performance: Application start order is deterministic and computed at compile-time. If you want to see the order, take a look at the end of the shoehorn.script in your release directory. Shoehorn alphabetizes the start of applications that could be ordered arbitrarily. This minimizes changes in start ordering when dependencies are added or removed. It enables experimental features like providing additional dependencies (using the :extra_dependencies configuration key) or hinting that dependencies get started as late as possible (the :last configuration key) You can remove the :app configuration key from your Shoehorn configuration. It's no longer needed. Aside from the change from a macro to a behaviour and possibly needing to implement callback functions, Shoehorn.Handler implementations work the same as before.","ref":"changelog.html#v0-9-0-2022-04-03","title":"Changelog - v0.9.0 - 2022-04-03","type":"extras"},{"doc":"Shoehorn v0.8.0 completely removes support for Distillery.","ref":"changelog.html#v0-8-0-2021-10-31","title":"Changelog - v0.8.0 - 2021-10-31","type":"extras"},{"doc":"Shoehorn 0.7.0 removes support for creating boot scripts using Distillery and only supports using Elixir releases. As a result, the minimum supported version of Elixir is now version 1.9.","ref":"changelog.html#v0-7-0","title":"Changelog - v0.7.0","type":"extras"},{"doc":"Enhancements Added support for Elixir 1.9+ releases. Distillery is now an optional dependency and ~&gt; 2.1. Updated supervisor specs and cleaned up warnings.","ref":"changelog.html#v0-6-0","title":"Changelog - v0.6.0","type":"extras"},{"doc":"Enhancements Exclude distillery, artificery, and mix from the release by default. Removed RPC and application overlay modules. Updated deps and docs.","ref":"changelog.html#v0-5-0","title":"Changelog - v0.5.0","type":"extras"},{"doc":"Enhancements Support for distillery ~&gt; 2.0 Support for Elixir ~&gt; 1.7","ref":"changelog.html#v0-4-0","title":"Changelog - v0.4.0","type":"extras"},{"doc":"For Shoehorn, these are our goals: 1: Fix current issue with prod devices turning to zombies 2: Make it really really difficult to enter a brick state ever. It was becoming apparent that it is difficult to address goal #1 by changing the defaults without impacting goal #2 at all. We believe that its best to solve goal #1 by opting in and not by modifying the defaults. In this release, existing projects that do not declare a handler in the config will use Shoehorn.Handler.Ignore .","ref":"changelog.html#v0-3-1","title":"Changelog - v0.3.1","type":"extras"},{"doc":"The default strategy for how Shoehorn handles OTP application exits has changed. Before this release, if an application were to exit the node would remain running and that applications would remain stopped. This may be desireable for development and test but is typically undesirable in production. This behaviour can be customized by configuring the handler in the config. For example, in dev you can use the module Shoehorn.Handler.Ignore to prevent the node from halting on failure. # config/dev.exs config :shoehorn , handler : Shoehorn.Handler.Ignore Check out the example application for information on implementing custom strategies.","ref":"changelog.html#v0-3-0","title":"Changelog - v0.3.0","type":"extras"},{"doc":"Renamed project Shoehorn. It became hard to discuss this project with the name Bootloader. Enhancements :init list can contain :application , {m, f, a} , or {Module, [args]} .","ref":"changelog.html#v0-2-0","title":"Changelog - v0.2.0","type":"extras"},{"doc":"Bug Fixes Add explicit functions for each of the Distillery Plugin behaviour callbacks.","ref":"changelog.html#v0-1-3","title":"Changelog - v0.1.3","type":"extras"},{"doc":"Bug Fixes Only look in :code.lib_dir() for the Application lib dir instead of involving mix Enhancements Warn when an app listed in :init or :app does not exist. Output message about booting using shoehorn during mix release","ref":"changelog.html#v0-1-2","title":"Changelog - v0.1.2","type":"extras"},{"doc":"Bug Fixes Fixed issue with release path being constructed incorrectly.","ref":"changelog.html#v0-1-1","title":"Changelog - v0.1.1","type":"extras"},{"doc":"Initial release to hex.","ref":"changelog.html#v0-1-0","title":"Changelog - v0.1.0","type":"extras"}]